@@ -549,13 +549,21 @@ class Workflow(FWSerializable):
             for k, v in list(self.items()):
                 if not isinstance(v, (list, tuple)):
                     self[k] = [v]  # v must be list
+
+                self[k] = [x.fw_id if hasattr(x, "fw_id") else x for x in self[k]]
+
                 if not isinstance(k, int):
-                    try:
-                        self[int(k)] = self[k]  # k must be int
-                    except:
-                        pass  # garbage input
+                    if hasattr(k, "fw_id"):  # maybe it's a String?
+                        self[k.fw_id] = self[k]
+                    else:  # maybe it's a String?
+                        try:
+                            self[int(k)] = self[k]  # k must be int
+                        except:
+                            pass  # garbage input
                     del self[k]
 
+
+
         @property
         def nodes(self):
             allnodes = list(self.keys())
@@ -623,7 +631,7 @@ class Workflow(FWSerializable):
                 raise ValueError('FW ids must be unique!')
             self.id_fw[fw.fw_id] = fw
 
-            if fw.fw_id not in links_dict:
+            if fw.fw_id not in links_dict and fw not in links_dict:
                 links_dict[fw.fw_id] = []
 
         self.links = Workflow.Links(links_dict)
